@@ -1,4 +1,4 @@
-/* $Id: etc.c,v 1.12 2001/12/10 15:42:38 ukai Exp $ */
+/* $Id: etc.c,v 1.16 2001/12/26 18:29:33 ukai Exp $ */
 #include "fm.h"
 #include <pwd.h>
 #include "myctype.h"
@@ -14,7 +14,9 @@
 
 #include <sys/types.h>
 #include <time.h>
+#if defined(HAVE_WAITPID) || defined(HAVE_WAIT3)
 #include <sys/wait.h>
+#endif
 #include <signal.h>
 
 #ifdef	__WATT32__
@@ -548,32 +550,6 @@ lastFileName(char *path)
     return allocStr(q, -1);
 }
 
-#ifndef HAVE_BCOPY
-void
-bcopy(void *src, void *dest, int len)
-{
-    int i;
-    if (src == dest)
-	return;
-    if (src < dest) {
-	for (i = len - 1; i >= 0; i--)
-	    dest[i] = src[i];
-    }
-    else {			/* src > dest */
-	for (i = 0; i < len; i++)
-	    dest[i] = src[i];
-    }
-}
-
-void
-bzero(void *ptr, int len)
-{
-    int i;
-    for (i = 0; i < len; i++)
-	*(ptr++) = 0;
-}
-#endif				/* not HAVE_BCOPY */
-
 #ifdef USE_INCLUDED_SRAND48
 static unsigned long R1 = 0x1234abcd;
 static unsigned long R2 = 0x330e;
@@ -912,39 +888,40 @@ correct_irrtag(int status)
 
 /* authentication */
 struct auth_cookie *
-find_auth(char *host, char *realm)
+find_auth(char *host, int port, char *realm)
 {
     struct auth_cookie *p;
 
     for (p = Auth_cookie; p != NULL; p = p->next) {
 	if (!Strcasecmp_charp(p->host, host) &&
-	    !Strcasecmp_charp(p->realm, realm))
+	    p->port == port && !Strcasecmp_charp(p->realm, realm))
 	    return p;
     }
     return NULL;
 }
 
 Str
-find_auth_cookie(char *host, char *realm)
+find_auth_cookie(char *host, int port, char *realm)
 {
-    struct auth_cookie *p = find_auth(host, realm);
+    struct auth_cookie *p = find_auth(host, port, realm);
     if (p)
 	return p->cookie;
     return NULL;
 }
 
 void
-add_auth_cookie(char *host, char *realm, Str cookie)
+add_auth_cookie(char *host, int port, char *realm, Str cookie)
 {
     struct auth_cookie *p;
 
-    p = find_auth(host, realm);
+    p = find_auth(host, port, realm);
     if (p) {
 	p->cookie = cookie;
 	return;
     }
     p = New(struct auth_cookie);
     p->host = Strnew_charp(host);
+    p->port = port;
     p->realm = Strnew_charp(realm);
     p->cookie = cookie;
     p->next = Auth_cookie;
@@ -1082,6 +1059,68 @@ mySystem(char *command, int background)
 	system(command);
 }
 
+Str
+myExtCommand(char *cmd, char *arg, int redirect)
+{
+    Str tmp = NULL;
+    char *p;
+    int set_arg = FALSE;
+
+    for (p = cmd; *p; p++) {
+	if (*p == '%' && *(p + 1) == 's' && !set_arg) {
+	    if (tmp == NULL)
+		tmp = Strnew_charp_n(cmd, (int)(p - cmd));
+	    Strcat_charp(tmp, arg);
+	    set_arg = TRUE;
+	    p++;
+	}
+	else {
+	    if (tmp)
+		Strcat_char(tmp, *p);
+	}
+    }
+    if (!set_arg)
+	tmp = Strnew_m_charp(cmd, (redirect ? " < " : " "), arg, NULL);
+    return tmp;
+}
+
+Str
+myEditor(char *cmd, char *file, int line)
+{
+    Str tmp = NULL;
+    char *p;
+    int set_file = FALSE, set_line = FALSE;
+
+    for (p = cmd; *p; p++) {
+	if (*p == '%' && *(p + 1) == 's' && !set_file) {
+	    if (tmp == NULL)
+		tmp = Strnew_charp_n(cmd, (int)(p - cmd));
+	    Strcat_charp(tmp, file);
+	    set_file = TRUE;
+	    p++;
+	}
+	else if (*p == '%' && *(p + 1) == 'd' && !set_line && line > 0) {
+	    if (tmp == NULL)
+		tmp = Strnew_charp_n(cmd, (int)(p - cmd));
+	    Strcat(tmp, Sprintf("%d", line));
+	    set_line = TRUE;
+	    p++;
+	}
+	else {
+	    if (tmp)
+		Strcat_char(tmp, *p);
+	}
+    }
+    if (!set_file) {
+	if (tmp == NULL)
+	    tmp = Strnew_charp(cmd);
+	if (!set_line && line > 0 && strcasestr(cmd, "vi"))
+	    Strcat(tmp, Sprintf(" +%d", line));
+	Strcat_m_charp(tmp, " ", file, NULL);
+    }
+    return tmp;
+}
+
 char *
 expandName(char *name)
 {
