@@ -1,4 +1,4 @@
-/* $Id: fm.h,v 1.29 2001/12/14 17:35:08 ukai Exp $ */
+/* $Id: fm.h,v 1.36 2001/12/27 17:50:56 ukai Exp $ */
 /* 
  * w3m: WWW wo Miru utility
  * 
@@ -47,7 +47,7 @@
 #include "terms.h"
 
 #ifndef HAVE_BCOPY
-void bcopy(void *, void *, int);
+void bcopy(const void *, void *, int);
 void bzero(void *, int);
 #endif				/* HAVE_BCOPY */
 #ifdef __EMX__
@@ -184,6 +184,11 @@ void bzero(void *, int);
 #define LB_N_SOURCE	LB_SOURCE
 #define MAX_LB		5
 
+/* Search Result */
+#define SR_FOUND       0x1
+#define SR_NOTFOUND    0x2
+#define SR_WRAPPED     0x4
+
 #ifdef MAINPROGRAM
 int REV_LB[MAX_LB] = {
     LB_N_FRAME, LB_FRAME, LB_N_INFO, LB_INFO, LB_N_SOURCE,
@@ -224,6 +229,7 @@ extern int REV_LB[];
  * Macros.
  */
 
+#define inputLineHist(p,d,f,h)	inputLineHistSearch(p,d,f,h,NULL)
 #define inputLine(p,d,f)	inputLineHist(p,d,f,NULL)
 #define inputStr(p,d)		inputLine(p,d,IN_STRING)
 #define inputStrHist(p,d,h)	inputLineHist(p,d,IN_STRING,h)
@@ -373,6 +379,21 @@ typedef struct _Buffer {
 #endif
 } Buffer;
 
+
+#define COPY_BUFPOSITION(dstbuf, srcbuf) {\
+ (dstbuf)->topLine = (srcbuf)->topLine; \
+ (dstbuf)->currentLine = (srcbuf)->currentLine; \
+ (dstbuf)->pos = (srcbuf)->pos; \
+ (dstbuf)->cursorX = (srcbuf)->cursorX; \
+ (dstbuf)->cursorY = (srcbuf)->cursorY; \
+ (dstbuf)->visualpos = (srcbuf)->visualpos; \
+ (dstbuf)->currentColumn = (srcbuf)->currentColumn; \
+}
+#define SAVE_BUFPOSITION(sbufp) COPY_BUFPOSITION(sbufp, Currentbuf)
+#define RESTORE_BUFPOSITION(sbufp) COPY_BUFPOSITION(Currentbuf, sbufp)
+#define TOP_LINENUMBER(buf) ((buf)->topLine ? (buf)->topLine->linenumber : 1)
+#define CUR_LINENUMBER(buf) ((buf)->currentLine ? (buf)->currentLine->linenumber : 1)
+
 #define NO_BUFFER ((Buffer*)1)
 
 #define RB_STACK_SIZE 10
@@ -518,6 +539,7 @@ struct html_feed_environ {
 
 struct auth_cookie {
     Str host;
+    int port;
     Str realm;
     Str cookie;
     struct auth_cookie *next;
@@ -752,6 +774,7 @@ global Str proxy_auth_cookie init(NULL);
 global int UseExternalDirBuffer init(TRUE);
 global char *DirBufferCommand init("file:///$LIB/dirlist" CGI_EXTENSION);
 global int ignore_null_img_alt init(TRUE);
+global int FoldTextarea init(FALSE);
 
 global struct auth_cookie *Auth_cookie init(NULL);
 global char *Local_cookie init(NULL);
@@ -858,7 +881,7 @@ global int use_lessopen init(FALSE);
 global int FollowRedirection init(10);
 
 global int w3m_backend init(FALSE);
-global Str backend_halfdump_str;
+global TextLineList *backend_halfdump_buf;
 global TextList *backend_batch_commands init(NULL);
 int backend(void);
 extern void deleteFiles(void);
