@@ -1,4 +1,4 @@
-
+/* $Id: history.h,v 1.4 2001/12/10 17:02:44 ukai Exp $ */
 #ifndef HISTORY_H
 #define HISTORY_H
 
@@ -13,8 +13,8 @@ typedef GeneralList HistList;
 
 typedef struct {
     HistList *list;
-    HistItem *current;   
-    Hash_hist *hash;   
+    HistItem *current;
+    Hash_sv *hash;
 } Hist;
 
 extern Hist *newHist();
@@ -26,4 +26,4 @@ extern char *lastHist(Hist *hist);
 extern char *nextHist(Hist *hist);
 extern char *prevHist(Hist *hist);
 
-#endif /* HISTORY_H */
+#endif				/* HISTORY_H */
