@@ -1,4 +1,4 @@
-/* $Id: url.c,v 1.22 2001/12/07 07:58:07 ukai Exp $ */
+/* $Id: url.c,v 1.28 2001/12/27 18:22:59 ukai Exp $ */
 #include "fm.h"
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -239,6 +239,7 @@ free_ssl_ctx()
 {
     if (ssl_ctx != NULL)
 	SSL_CTX_free(ssl_ctx);
+    ssl_ctx = NULL;
 }
 
 #if SSLEAY_VERSION_NUMBER >= 0x00905100
@@ -272,11 +273,14 @@ init_PRNG()
 #endif				/* SSLEAY_VERSION_NUMBER >= 0x00905100 */
 
 static SSL *
-openSSLHandle(int sock)
+openSSLHandle(int sock, char *hostname)
 {
-    SSL *handle;
-    char *emsg;
+    SSL *handle = NULL;
+    Str emsg;
+    Str amsg = NULL;
+    char *ans;
     static char *old_ssl_forbid_method = NULL;
+    static Str accept_this_site = NULL;
 #ifdef USE_SSL_VERIFY
     static int old_ssl_verify_server = -1;
 #endif
@@ -288,7 +292,6 @@ openSSLHandle(int sock)
 	ssl_path_modified = 1;
 #else
 	free_ssl_ctx();
-	ssl_ctx = NULL;
 #endif
     }
 #ifdef USE_SSL_VERIFY
@@ -298,7 +301,7 @@ openSSLHandle(int sock)
     }
     if (ssl_path_modified) {
 	free_ssl_ctx();
-	ssl_ctx = NULL;
+	accept_this_site = NULL;
 	ssl_path_modified = 0;
     }
 #endif				/* defined(USE_SSL_VERIFY) */
@@ -326,9 +329,13 @@ openSSLHandle(int sock)
 	SSL_CTX_set_options(ssl_ctx, option);
 #ifdef USE_SSL_VERIFY
 	/* derived from openssl-0.9.5/apps/s_{client,cb}.c */
+#if 1				/* use SSL_get_verify_result() to verify cert */
+	SSL_CTX_set_verify(ssl_ctx, SSL_VERIFY_NONE, NULL);
+#else
 	SSL_CTX_set_verify(ssl_ctx,
 			   ssl_verify_server ? SSL_VERIFY_PEER :
 			   SSL_VERIFY_NONE, NULL);
+#endif
 	if (ssl_cert_file != NULL && *ssl_cert_file != '\0') {
 	    int ng = 1;
 	    if (SSL_CTX_use_certificate_file
@@ -350,9 +357,6 @@ openSSLHandle(int sock)
 #endif				/* defined(USE_SSL_VERIFY) */
 	    SSL_CTX_set_default_verify_paths(ssl_ctx);
 #endif				/* SSLEAY_VERSION_NUMBER >= 0x0800 */
-/*** by inu
-        atexit(free_ssl_ctx);
-*/
     }
     handle = SSL_new(ssl_ctx);
     SSL_set_fd(handle, sock);
@@ -361,11 +365,103 @@ openSSLHandle(int sock)
 #endif				/* SSLEAY_VERSION_NUMBER >= 0x00905100 */
     if (SSL_connect(handle) <= 0)
 	goto eend;
+#ifdef USE_SSL_VERIFY
+    /* check the cert chain.
+     * The chain length is automatically checked by OpenSSL when we
+     * set the verify depth in the ctx.
+     */
+    if (ssl_verify_server) {
+	X509 *x;
+	x = SSL_get_peer_certificate(handle);
+	if (x == NULL) {
+	    if (accept_this_site
+		&& strcasecmp(accept_this_site->ptr, hostname) == 0)
+		ans = "y";
+	    else {
+		emsg = Strnew_charp("No SSL peer certificate: accept (y/n)?");
+		term_raw();
+		ans = inputChar(emsg->ptr);
+	    }
+	    if (tolower(*ans) == 'y')
+		amsg =
+		    Strnew_charp
+		    ("Accept SSL session without any peer certificate");
+	    else {
+		char *e = "This SSL session was rejected "
+		    "to prevent security violation: no peer certificate";
+		disp_err_message(e, FALSE);
+		free_ssl_ctx();
+		return NULL;
+	    }
+	}
+	else {
+	    long verr;
+	    X509_free(x);
+	    if ((verr = SSL_get_verify_result(handle)) != X509_V_OK) {
+		const char *em = X509_verify_cert_error_string(verr);
+		if (accept_this_site
+		    && strcasecmp(accept_this_site->ptr, hostname) == 0)
+		    ans = "y";
+		else {
+		    emsg = Sprintf("%s: accept (y/n)?", em);
+		    term_raw();
+		    ans = inputChar(emsg->ptr);
+		}
+		if (tolower(*ans) == 'y') {
+		    amsg = Sprintf("Accept unsecure SSL session: "
+				   "unverified: %s", em);
+		}
+		else {
+		    char *e =
+			Sprintf("This SSL session was rejected: %s", em)->ptr;
+		    disp_err_message(e, FALSE);
+		    free_ssl_ctx();
+		    return NULL;
+		}
+	    }
+	}
+    }
+    else
+#endif
+	amsg = Strnew_charp("Certificate is not verified");
+
+    emsg = ssl_check_cert_ident(handle, hostname);
+    if (emsg != NULL) {
+	if (accept_this_site
+	    && strcasecmp(accept_this_site->ptr, hostname) == 0)
+	    ans = "y";
+	else {
+	    Str ep = Strdup(emsg);
+	    if (ep->length > COLS - 16)
+		Strshrink(ep, ep->length - (COLS - 16));
+	    term_raw();
+	    Strcat_charp(ep, ": accept(y/n)?");
+	    ans = inputChar(ep->ptr);
+	}
+	if (tolower(*ans) == 'y') {
+	    amsg = Strnew_charp("Accept unsecure SSL session:");
+	    Strcat(amsg, emsg);
+	}
+	else {
+	    char *e = "This SSL session was rejected "
+		"to prevent security violation";
+	    disp_err_message(e, FALSE);
+	    free_ssl_ctx();
+	    return NULL;
+	}
+    }
+    ssl_set_certificate_validity(amsg);
+    if (amsg)
+	disp_err_message(amsg->ptr, FALSE);
+    accept_this_site = Strnew_charp(hostname);
     return handle;
   eend:
-    emsg =
-	Sprintf("SSL error: %s", ERR_error_string(ERR_get_error(), NULL))->ptr;
-    disp_err_message(emsg, FALSE);
+    close(sock);
+    if (handle)
+	SSL_free(handle);
+    accept_this_site = NULL;
+    emsg = Sprintf("SSL error: %s", ERR_error_string(ERR_get_error(), NULL));
+    disp_err_message(emsg->ptr, FALSE);
     return NULL;
 }
 
@@ -1522,7 +1618,7 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 #ifdef USE_SSL
 	    if (pu->scheme == SCM_HTTPS && *status == HTST_CONNECT) {
 		sock = ssl_socket_of(ouf->stream);
-		if (!(sslh = openSSLHandle(sock))) {
+		if (!(sslh = openSSLHandle(sock, pu->host))) {
 		    *status = HTST_MISSING;
 		    return uf;
 		}
@@ -1576,7 +1672,7 @@ openURL(char *url, ParsedURL *pu, ParsedURL *current,
 	    }
 #ifdef USE_SSL
 	    if (pu->scheme == SCM_HTTPS) {
-		if (!(sslh = openSSLHandle(sock))) {
+		if (!(sslh = openSSLHandle(sock, pu->host))) {
 		    *status = HTST_MISSING;
 		    return uf;
 		}
